#pragma option v;
#include <17c42.h>

/*
   These routines may be adapted for any purpose when
   used with the MPC Code Development system.  No
   warranty is implied or given as to their usability
   for any purpose.

        (c) Copyright 1995

        Byte Craft Limited
        Waterloo, Ontario
        Canada N2J 4E4
        (519) 888-6911

        Walter Banks

   In the PIC17CXX, variables can be stored in the program memory.
   If external RAM is connected and the 17CXX is used in either
   extended microcontroller mode, or microprocessor mode; then
   variables can be stored in an address space of up to 64K words.

   In assembler, the four instructions TABLRD, TABLWT, TLRD and
   TLWT are used to read/write variables in program memory.
   In MPC far pointers are used. By default pointers are 'near'
   and hence they can only access variables in the register file.
   Far pointers, however, point to the program memory address
   space.

   To write a 16-bit (long) variable into program memory simply
   declare a 'long far' pointer, assign it a certain address
   value, then use pointer indirection to write the value.
   e.g.
        long l=0xABCD;
        long far *p;
        p = 0x1234;
        *p = l;
     // Location 0x1234 now contains  0xABCD

   To write an 8-bit variable into the LSB of a program memory
   location, use the same procedure as above. The compiler
   will write the variable in the LSB and will write a 0 in the
   MSB of the memory location;
   e.g.
         int i=0xAA;
         *p = i;    
     // Location 0x1234 now contains   0x00AA

   To write an 8-bit variable in the LSB of a program memory
   location, declare an 8-bit far pointer, assign it to the
   same address as the 'long' pointer and then write the variable
   to that pointer.
   e.g.
        char far *q;
        q = p;
        char c='A';  // ASCII 0x41
        *q = c;
      // Location 0x1234 now contains  0x41AA

   To read/write a series of numbers contiguously in memory
   simply increment the pointer.
   e.g.
        for (i=0 ; i<50 ; i++)
          {
           PORTB = *p;  // LSB
           PORTC = *q;  // MSB
           p++;
           q++;
          }
*/

long far *p;  // declation to access both ROM bytes
char far *q;  // declaration to access MSB

long l;
char c;
int i;

void main (void)
 {
   p = 0x1234;   // Access location 0x1234
   q = p;        // Assign 'q' to same ROM location

   i = *p;       // Read LSB of 0x1234
   c = *q;       // Read MSB of 0x1234
   l = *p;       // Read the entire word

   *p = i;       // Write LSB, MSB=0
   *q = c;       // Write MSB
   *p = l;       // Write entire word

   // Reading a series of locations
   DDRB = 0x00;  // PORTB in output mode
   DDRC = 0x00;  // PORTC in output mode
   for (i=0 ; i<50 ; i++)
    {
     PORTB = *p;  // LSB
     PORTC = *q;  // MSB
     p++;
     q++;
    }
  } // main()

